# frozen_string_literal: true

describe BridgeAPI::Client do
  it 'should set the auth header to basic auth' do
    client = BridgeAPI::Client.new(prefix: 'https://www.fake.com', api_key: 'fake_token', api_secret: 'fake_secret')
    expect(client.connection.headers['Authorization']).to(eq('Basic ZmFrZV90b2tlbjpmYWtlX3NlY3JldA=='))
  end

  it 'should set the auth header to bearer auth' do
    client = BridgeAPI::Client.new(prefix: 'https://www.fake.com', token: 'test_token')
    expect(client.connection.headers['Authorization']).to(eq('Bearer test_token'))
  end

  it 'should set the meta property on client' do
    client = BridgeAPI::Client.new(prefix: 'https://www.fake.com', token: 'test_token')
    users  = client.get_all_users
    expect(users.meta['next']).to(eq('http://bridge.bridgeapp.com/api/author/users?after=eyJ0eXAiOiJKV1QiLCJhSDiQQ'))
  end

  it 'should set the linked property on client' do
    client = BridgeAPI::Client.new(prefix: 'https://www.fake.com', token: 'test_token')
    users  = client.get_all_users
    expect(users.linked['custom_fields'][0]['id']).to(eq('1'))
  end

  it 'should enforce rate limits' do
    client                        = BridgeAPI::Client.new(prefix: 'https://www.fake.com', api_key: 'fake_token', api_secret: 'fake_secret')
    BridgeAPI.enforce_rate_limits = true
    users                         = client.get_all_users
    users                         = client.get_all_users
  end

  describe 'initialize' do
    it 'should set initial token when using a pool' do
      expect_any_instance_of(BridgeAPI::Client).to receive(:initialize_from_token_pool).and_call_original
      client = BridgeAPI::Client.new(prefix: 'https://www.fake.com', api_keys: { 'key1' => 'secret1', 'key2' => 'secret2' })
      expect(client.config[:api_key]).to eq('key1')
      expect(client.config[:api_secret]).to eq('secret1')
    end

    it 'should not call initialize_from_token_pool for a single token' do
      expect_any_instance_of(BridgeAPI::Client).to_not receive(:initialize_from_token_pool)
      BridgeAPI::Client.new(prefix: 'https://www.fake.com', api_key: 'fake_token', api_secret: 'fake_secret')
    end
  end

  describe 'has_token_pool?' do
    it 'should return true for api keys' do
      config = { prefix: 'https://www.fake.com', api_keys: { 'key1' => 'secret1', 'key2' => 'secret2' } }
      client = BridgeAPI::Client.new(config)
      expect(client.has_token_pool?(config)).to be_truthy
    end

    it 'should return true for tokens' do
      config = { prefix: 'https://www.fake.com', api_tokens: %w[dGVzdDE6dGVzdDE= dGVzdDI6dGVzdDI=] }
      client = BridgeAPI::Client.new(config)
      expect(client.has_token_pool?(config)).to be_truthy
    end

    it 'should return false for single key' do
      config = { prefix: 'https://www.fake.com', api_key: 'fake_token', api_secret: 'fake_secret' }
      client = BridgeAPI::Client.new(config)
      expect(client.has_token_pool?(config)).to be_falsey
    end
  end

  describe 'initialize_from_token_pool' do
    it 'should take first from api keys' do
      config     = { prefix: 'https://www.fake.com', api_keys: { 'key1' => 'secret1', 'key2' => 'secret2' } }
      client     = BridgeAPI::Client.new(config)
      new_config = client.initialize_from_token_pool(config)
      expect(new_config[:api_key]).to eq('key1')
      expect(new_config[:api_secret]).to eq('secret1')
    end

    it 'should take first from tokens' do
      config     = { prefix: 'https://www.fake.com', api_tokens: %w[dGVzdDE6dGVzdDE= dGVzdDI6dGVzdDI=] }
      client     = BridgeAPI::Client.new(config)
      new_config = client.initialize_from_token_pool(config)
      expect(new_config[:api_key]).to eq('test1')
    end
  end

  describe 'get_next_key' do
    it 'should find next key' do
      keys   = { 'key1' => 'secret1', 'key2' => 'secret2' }
      config = { prefix: 'https://www.fake.com', api_keys: keys }
      client = BridgeAPI::Client.new(config)
      expect(client.get_next_key(keys.keys, 'key1')).to eq('key2')
      expect(client.get_next_key(keys.keys, 'key2')).to eq('key1')
      expect(client.get_next_key(keys.keys, 'non-existent-key')).to eq('key1')
    end
  end

  describe 'rotate_token!' do
    context 'with api key pool' do
      it 'should rotate to the next key' do
        keys   = { 'key1' => 'secret1', 'key2' => 'secret2' }
        config = { prefix: 'https://www.fake.com', api_keys: keys }
        client = BridgeAPI::Client.new(config)
        expect(client.config[:api_key]).to eq('key1')
        expect(client.config[:api_secret]).to eq('secret1')
        client.rotate_token!
        expect(client.config[:api_key]).to eq('key2')
        expect(client.config[:api_secret]).to eq('secret2')
        client.rotate_token!
        expect(client.config[:api_key]).to eq('key1')
        expect(client.config[:api_secret]).to eq('secret1')
      end
    end

    context 'with token pool' do
      it 'should rotate to the next key' do
        keys   = %w[dGVzdDE6dGVzdDE= dGVzdDI6dGVzdDI=]
        config = { prefix: 'https://www.fake.com', api_tokens: keys }
        client = BridgeAPI::Client.new(config)
        expect(client.config[:api_key]).to eq('test1')
        client.rotate_token!
        expect(client.config[:api_key]).to eq('test2')
        client.rotate_token!
        expect(client.config[:api_key]).to eq('test1')
      end
    end
  end
end

